package de.dhbwloe.campusapp;

import android.content.Context;
import android.content.Intent;
import android.nfc.NfcAdapter;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.support.design.widget.NavigationView;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.util.Date;

import de.dhbwloe.campusapp.nfcreader.NfcCardListener;

public class CampusApp extends FragmentActivity {

    private boolean bSearchActive = false;
    private CampusAppContext AppContext = null;

    /*
    * Dev Info:
    *
    * Die App besteht aus einer einzigen Activity, auf welcher dynamisch Fragmente platziert werden.
    * Das Menü, sowie die Headerleiste befinden sich auf der Activity, sind dementsprechend immer verfügbar.
    *
    * Zur laufzeit der App wird eine einzige Instanz der Klasse CampusAppContext angelegt.
    * Dieser Kontext dient als einstiegspunkt für sämmtliche global verfügbaren verwaltungs tools.
    * Dazu gehören:
    *   AppContext.getNavigationManager()    NavigationManager   Verwaltet die angezeigten Fragmente und deren Navigation
    *                                                            .navigatePage("Dashboard")  to navigate to the Dashboard
    *   ... to be continued ...
     */

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        Log.i("CampusApp", "Event: onCreate");
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_campus_app);

        AppContext = CampusAppContext.getInstance();
        if(AppContext == null)
            AppContext = new CampusAppContext(this, R.id.fragment_container);
        else
            AppContext.setMainActivity(this);

        boolean instantRestore = false;
        if(savedInstanceState != null) {
            long lastrun = savedInstanceState.getLong("lastrun");
            lastrun = ((new Date()).getTime()/1000) - lastrun;
            Log.i("CampusApp", "Restored from Idle state! Idled: "+lastrun+" secs");
            if(lastrun < 30) {
                instantRestore = true;

                AppContext.setTitle(savedInstanceState.getString("activetitle"));
            }


            Bundle contextVars = savedInstanceState.getBundle("savedContextVariables");
            if(contextVars != null) {
                Bundle contextVariables = AppContext.getContextVariables();
                contextVariables.putAll(contextVars);
            }
        }

        prepareMainUi();
        if(instantRestore) // orientation change
            loadMainUi();
        else
            // don't add navigation to SlashScreen, or even from SplashScreen to Dashboard to the history log!
            // it would be very strange to return to the SplashScreen or an "empty" page on return ;)
            AppContext.getNavigationManager().navigatePage("SplashScreen", null, false);

    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {
        Log.i("CampusApp", "Event: onSaveInstanceState");
        // Save instance state during "restarts" due to orientation changes.
        // We don't want to see the splash screen everytime the orientation changes ;)
        savedInstanceState.putLong("lastrun", (new Date()).getTime() / 1000);
        savedInstanceState.putString("activepage", AppContext.getNavigationManager().getCurrentPageName());
        TextView titleView = (TextView) findViewById(R.id.title);
        savedInstanceState.putString("activetitle", titleView.getText().toString());

        Bundle contextVariables = AppContext.getContextVariables();
        savedInstanceState.putBundle("savedContextVariables", contextVariables);

        // Always call the superclass so it can save the view hierarchy state
        super.onSaveInstanceState(savedInstanceState);

        AppContext = null; // simulate GC
    }

    public void prepareMainUi() {
        ImageView btnOpenSearch = (ImageView) findViewById(R.id.search_button);
        btnOpenSearch.setVisibility(View.GONE);
    }

    public void loadMainUi() {
        ImageView btnOpenSearch = (ImageView) findViewById(R.id.search_button);

        btnOpenSearch.setVisibility(View.VISIBLE);
        setupActionBar();
        setupSearchTriggers();

        AppContext.getNfcCardListener().startNfcListener();
    }

    private void setupActionBar() {
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.setDrawerListener(toggle);
        toggle.syncState();

        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(new NavigationView.OnNavigationItemSelectedListener() {
            @SuppressWarnings("StatementWithEmptyBody")
            @Override
            public boolean onNavigationItemSelected(MenuItem item) {
                // Handle navigation view item clicks here.
                int id = item.getItemId();

                switch (id) { // Navigation Items from res/menu/activity_campus_app_drawer.xml
                    case R.id.nav_dashboard:
                        AppContext.getNavigationManager().navigatePage("Dashboard");
                        break;
                    case R.id.nav_vorlesungsplan:
                        AppContext.getNavigationManager().navigatePage("Vorlesungsplan");
                        break;
                    case R.id.nav_mensa:
                        AppContext.getNavigationManager().navigatePage("Mensa");
                        break;
                    case R.id.nav_news:
                        AppContext.getNavigationManager().navigatePage("News");
                        break;
                    case R.id.nav_settings:
                        AppContext.getNavigationManager().navigatePage("Settings");
                        break;
                    case R.id.nav_wifi:
                        AppContext.getNavigationManager().navigatePage("WifiSettings");
                        break;
                    case R.id.nav_impressum:
                        AppContext.getNavigationManager().navigatePage("Impressum");
                        break;
                    default:
                        return false;
                }

                DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
                drawer.closeDrawer(GravityCompat.START);
                return true;
            }
        });
    }

    private void setupSearchTriggers() {
        ImageView btnOpenSearch = (ImageView) findViewById(R.id.search_button);
        ImageView btnCloseSearch = (ImageView) findViewById(R.id.search_clear);
        EditText edtSearchInput = (EditText) findViewById(R.id.search_input);
        TextView txtTitle = (TextView) findViewById(R.id.title);

        btnOpenSearch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                triggerSearchPanel(true);
            }
        });
        edtSearchInput.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                EditText edtSearchInput = (EditText) v;
                if (!hasFocus && edtSearchInput.getText().length() == 0) {
                    triggerSearchPanel(false);
                }
            }
        });
        edtSearchInput.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                if (actionId == EditorInfo.IME_ACTION_SEARCH) {
                    EditText edtSearchInput = (EditText) v;
                    triggerSearchAction(edtSearchInput.getText().toString());
                    triggerSearchPanel(false);
                    return true;
                }
                return false;
            }
        });
        btnCloseSearch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                EditText edtSearchInput = (EditText) findViewById(R.id.search_input);
                edtSearchInput.setText("");
                triggerSearchPanel(false);
            }
        });
        txtTitle.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                triggerSearchPanel(true);
            }
        });

    }

    private void triggerSearchPanel(boolean show) {
        if(bSearchActive == show)
            return;
        bSearchActive = show;
        LinearLayout layTitleContainer = (LinearLayout) findViewById(R.id.title_container);
        LinearLayout laySearchContainer = (LinearLayout) findViewById(R.id.search_container);
        EditText edtSearchInput = (EditText) findViewById(R.id.search_input);

        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        if(show) {
            layTitleContainer.setVisibility(View.GONE);
            laySearchContainer.setVisibility(View.VISIBLE);
            edtSearchInput.requestFocus();

            imm.showSoftInput(edtSearchInput, InputMethodManager.SHOW_IMPLICIT);
        } else {
            layTitleContainer.setVisibility(View.VISIBLE);
            laySearchContainer.setVisibility(View.GONE);

            View view = this.getCurrentFocus();
            if (view != null) {
                imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
            }
        }
    }

    private void triggerSearchAction(String query) {
        Bundle bundle = new Bundle();
        bundle.putString("query", query);

        AppContext.getNavigationManager().navigatePage("AppSearch", bundle);
    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else if(!AppContext.getNavigationManager().back()) {
            super.onBackPressed(); // trigger system action if internal navigation manager returns false for back()
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.campus_app, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    /* nfc listener related callbacks */
    @Override
    public void onResume() {
        if(AppContext == null)
            AppContext = new CampusAppContext(this, R.id.fragment_container);
        else
            AppContext.setMainActivity(this);

        super.onResume();
        Log.i("CampusApp", "onResume event");
        AppContext.getNfcCardListener().resumeForefrontDispatcher();
    }

    @Override
    public void onPause() {
        super.onPause();
        Log.i("CampusApp", "Event: onPause");
        AppContext.getNfcCardListener().pauseForefrontDispatcher();
    }

    @Override
    public void onNewIntent(Intent intent) {
        if (NfcAdapter.ACTION_TECH_DISCOVERED.equals(intent.getAction())) {
            AppContext.getNfcCardListener().handleNfcEvent(intent);
        }
    }

}
