package de.dhbwloe.campusapp.fragments;


import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ListView;

import java.util.ArrayList;
import java.util.Date;

import de.dhbwloe.campusapp.CampusAppFragment;
import de.dhbwloe.campusapp.R;
import de.dhbwloe.campusapp.vorlesungen.CourseGroup;

/**
 * A simple {@link Fragment} subclass.
 */
public class VorlesungsplanGroups extends CampusAppFragment implements Vorlesungsplan.VorlesungsplanFragment {
    private String coursename;
    private View view;
    private VorlesungsplanGroupsListAdapter listAdapter;
    private ArrayList<VorlesungsplanGroupsListItem> listItems = new ArrayList<VorlesungsplanGroupsListItem>();
    private boolean showOldCourses = false;

    public VorlesungsplanGroups() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        String kursTag = AppContext.getDatabaseManager().getRuntimeCache("CourseName");
        if(kursTag == null || kursTag.isEmpty())
            return null;
        coursename = kursTag;

        view = inflater.inflate(R.layout.fragment_vorlesungsplan_groups, container, false);

        ListView listView = (ListView) view.findViewById(R.id.listView);
        listAdapter = new VorlesungsplanGroupsListAdapter(view.getContext(), R.layout.fragment_vorlesungsplan_groups_course, listItems);
        listView.setAdapter(listAdapter);

        CheckBox showOldChkBox = (CheckBox) view.findViewById(R.id.showOldCourses);
        showOldChkBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                showOldCourses = isChecked;
                updateCoursesList();
            }
        });

        return view;
    }

    @Override
    public void setActive() {

    }


    @Override
    public void onResume() {
        super.onResume();
        if(coursename != null)
            updateCoursesList();
    }

    private void updateCoursesList() {
        Date notBefore;
        if(showOldCourses)
            notBefore = null;
        else
            notBefore = new Date();

        CourseGroup[] groups = AppContext.getDatabaseManager().getCourseGroups(coursename, notBefore);
        listItems.clear();
        for(CourseGroup group : groups) {
            Bundle extraData = group.getExtraData();
            long nextKlausur = extraData.getLong("NextKlausurEvent");
            long lastEvent = extraData.getLong("LastEvent");

            // wenn nextKlausur über 5 jahrevon LastEvent entfernt liegt gibt es in diesem kurs KEINE Klausur.
            // der fiktive Wert musste zur technischen funktionalität der Datenbankabfrage eingefügt werden.
            if(nextKlausur - lastEvent > (86400 * 365 * 5))
                nextKlausur = 0;

            VorlesungsplanGroupsListItem item = new VorlesungsplanGroupsListItem(
                    group.getGroupId(),
                    coursename,
                    group.getGroupName(),
                    extraData.getLong("FirstEvent"),
                    lastEvent,
                    extraData.getInt("EventCount"),
                    extraData.getLong("NextEvent"),
                    nextKlausur
            );
            listItems.add(item);
        }
        if(listAdapter != null)
            listAdapter.notifyDataSetChanged();
    }

}
